from __future__ import annotations

from typing import Any
from typing import Union

import regex

from dsp_tools.models.exceptions import BaseError


class DateTimeStamp:
    """
    Class to hold and process an xsd:dateTimeStamp
    """

    _dateTimeStamp: str
    _validation_regex = (
        r"^-?([1-9][0-9]{3,}|0[0-9]{3})"
        r"-(0[1-9]|1[0-2])"
        r"-(0[1-9]|[12][0-9]|3[01])"
        r"T(([01][0-9]|2[0-3]):[0-5][0-9]:[0-5][0-9](\.[0-9]+)?|(24:00:00(\.0+)?))"
        r"(Z|(\+|-)((0[0-9]|1[0-3]):[0-5][0-9]|14:00))$"
    )

    def __init__(self, val: Any):
        """
        The constructor works for different inputs: a string, an instance of "DateTimeStamp",
        or a JSON-LD construct of the form { "@type": "xsd:dateTimeStamp", "@value": "date-str" }

        Args:
            val: xsd:dateTimeStamp as string, instance of "DateTimeStamp" or json-ld construct

        Raises:
            BaseError: if the JSON-LD construct is not correct
        """
        if isinstance(val, str):
            if not regex.search(self._validation_regex, val):
                raise BaseError(f"Invalid xsd:dateTimeStamp: '{val}'")
            self._dateTimeStamp = val
        elif isinstance(val, DateTimeStamp):
            self._dateTimeStamp = str(val)
        elif val.get("@type") == "xsd:dateTimeStamp" and regex.search(self._validation_regex, str(val.get("@value"))):
            self._dateTimeStamp = val["@value"]
        else:
            raise BaseError(f"Invalid xsd:dateTimeStamp: '{val}'")

    def __eq__(self, other: Union[str, "DateTimeStamp"]) -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp == other._dateTimeStamp

    def __lt__(self, other: "DateTimeStamp") -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp < other._dateTimeStamp

    def __le__(self, other: "DateTimeStamp") -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp <= other._dateTimeStamp

    def __gt__(self, other: "DateTimeStamp") -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp > other._dateTimeStamp

    def __ge__(self, other: "DateTimeStamp") -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp >= other._dateTimeStamp

    def __ne__(self, other: DateTimeStamp) -> bool:
        if isinstance(other, str):
            other = DateTimeStamp(other)
        return self._dateTimeStamp != other._dateTimeStamp

    def __str__(self: "DateTimeStamp") -> str:
        return self._dateTimeStamp

    def toJsonObj(self) -> dict[str, str]:
        return {"@type": "xsd:dateTimeStamp", "@value": self._dateTimeStamp}



import warnings
from warnings import WarningMessage

import pytest

from dsp_tools.models.custom_warnings import DspToolsUserWarning
from dsp_tools.xmllib.models.migration_metadata import MigrationMetadata


def test_migration_metadata_creation_date_good() -> None:
    with warnings.catch_warnings(record=True) as caught_warnings:
        MigrationMetadata("2019-01-09T15:45:54.502951Z", None, None, "id")
        assert len(caught_warnings) == 1
        assert isinstance(caught_warnings[0], WarningMessage)


def test_migration_metadata_iri_good() -> None:
    with warnings.catch_warnings(record=True) as caught_warnings:
        MigrationMetadata(None, "http://rdfh.ch/4123/TqAnYQzrSzC2ctT06OJMYB", None, "id")
        assert len(caught_warnings) == 1
        assert isinstance(caught_warnings[0], WarningMessage)


def test_migration_metadata_ark_good() -> None:
    with warnings.catch_warnings(record=True) as caught_warnings:
        MigrationMetadata(None, None, "ark:/72163/4123-43xc6ivb931-a.2022829", "id")
        assert len(caught_warnings) == 1
        assert isinstance(caught_warnings[0], WarningMessage)


def test_migration_metadata_creation_date_warns() -> None:
    with pytest.warns(DspToolsUserWarning):
        MigrationMetadata("2019-01-054.502951Z", None, None, "id")


def test_migration_metadata_iri_warns() -> None:
    with pytest.warns(DspToolsUserWarning):
        MigrationMetadata(None, "http:123/TqAnYQzrSzC2ctT06OJMYB", None, "id")


def test_migration_metadata_ark_warns() -> None:
    with pytest.warns(DspToolsUserWarning):
        MigrationMetadata(None, None, "163/4123-43xc6ivb931-a.2022829", "id")


def test_migration_metadata_as_attrib_empty() -> None:
    with pytest.warns(DspToolsUserWarning):
        result = MigrationMetadata(None, None, None, "id").as_attrib()
    assert not result


if __name__ == "__main__":
    pytest.main([__file__])



from typing import Any


def convert_to_bool_string(value: Any) -> str:
    """
    Turns a value into a bool string, suitable for an XML.

    Args:
        value: value to transform

    Returns:
        'true' or 'false' if it is a known value,
        else it returns the original value as a string.
    """
    str_val = str(value).lower().strip()
    if str_val in ("false", "0", "0.0", "no"):
        return "false"
    elif str_val in ("true", "1", "1.0", "yes"):
        return "true"
    return str(value)



from rdflib import RDF
from rdflib import RDFS
from rdflib import XSD
from rdflib import Literal
from rdflib import Namespace
from rdflib import URIRef

from dsp_tools.commands.xml_validate.models.data_rdf import BooleanValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import ColorValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import DateValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import DecimalValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import GeonameValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import IntValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import LinkValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import ListValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import ResourceRDF
from dsp_tools.commands.xml_validate.models.data_rdf import RichtextRDF
from dsp_tools.commands.xml_validate.models.data_rdf import SimpleTextRDF
from dsp_tools.commands.xml_validate.models.data_rdf import TimeValueRDF
from dsp_tools.commands.xml_validate.models.data_rdf import UriValueRDF

KNORA_API = Namespace("http://api.knora.org/ontology/knora-api/v2#")
API_SHAPES = Namespace("http://api.knora.org/ontology/knora-api/shapes/v2#")
ONTO = Namespace("http://0.0.0.0:3333/ontology/9999/onto/v2#")


class TestResource:
    def test_with_value(self, rdf_resource: ResourceRDF) -> None:
        g = rdf_resource.make_graph()
        assert len(g) == 2
        out_res_id = next(g.subjects(RDF.type, ONTO.ClassWithEverything))
        assert out_res_id == rdf_resource.res_iri
        lbl = next(g.objects(out_res_id, RDFS.label))
        assert lbl == Literal("lbl", datatype=XSD.string)


class TestBooleanValue:
    def test_make_graph_corr(self, rdf_boolean_value_corr: BooleanValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_boolean_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.BooleanValue))
        bn_prop = next(g.subjects(KNORA_API.booleanValueAsBoolean))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testBoolean, bn_type))
        assert res_id == in_id


class TestColorValue:
    def test_make_graph_corr(self, rdf_color_value_corr: ColorValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_color_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.ColorValue))
        bn_prop = next(g.subjects(KNORA_API.colorValueAsColor))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testColor, bn_type))
        assert res_id == in_id


class TestDateValueRDF:
    def test_make_graph_corr(self, rdf_date_value_corr: DateValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_date_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.DateValue))
        bn_prop = next(g.subjects(KNORA_API.valueAsString))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testSubDate1, bn_type))
        assert res_id == in_id


class TestDecimalValue:
    def test_make_graph_corr(self, rdf_decimal_value_corr: DecimalValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_decimal_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.DecimalValue))
        bn_prop = next(g.subjects(KNORA_API.decimalValueAsDecimal))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testDecimalSimpleText, bn_type))
        assert res_id == in_id


class TestGeonameValue:
    def test_make_graph_corr(self, rdf_geoname_value_corr: GeonameValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_geoname_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.GeonameValue))
        bn_prop = next(g.subjects(KNORA_API.geonameValueAsGeonameCode))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testGeoname, bn_type))
        assert res_id == in_id


class TestIntValue:
    def test_make_graph_corr(self, rdf_integer_value_corr: IntValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_integer_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.IntValue))
        bn_prop = next(g.subjects(KNORA_API.intValueAsInt))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testIntegerSimpleText, bn_type))
        assert res_id == in_id


class TestLinkValue:
    def test_make_graph_corr(self, rdf_link_value_corr: LinkValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_link_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.LinkValue))
        bn_prop = next(g.subjects(API_SHAPES.linkValueHasTargetID))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testHasLinkTo, bn_type))
        assert res_id == in_id


class TestListValue:
    def test_make_graph_corr(self, rdf_list_value_corr: ListValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_list_value_corr.make_graph()
        assert len(g) == 4
        bn_type = next(g.subjects(RDF.type, KNORA_API.ListValue))
        bn_prop = next(g.subjects(API_SHAPES.listNodeAsString))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testListProp, bn_type))
        assert res_id == in_id


class TestRichtext:
    def test_make_graph_corr(self, rdf_text_richtext_value_corr: RichtextRDF) -> None:
        in_id = URIRef("id")
        g = rdf_text_richtext_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.TextValue))
        bn_prop = next(g.subjects(KNORA_API.textValueAsXml))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testRichtext, bn_type))
        assert res_id == in_id


class TestSimpleText:
    def test_make_graph_corr(self, rdf_text_simpletext_value_corr: SimpleTextRDF) -> None:
        in_id = URIRef("id")
        g = rdf_text_simpletext_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.TextValue))
        bn_prop = next(g.subjects(KNORA_API.valueAsString))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testTextarea, bn_type))
        assert res_id == in_id


class TestTimeValue:
    def test_make_graph_corr(self, rdf_time_value_corr: TimeValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_time_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.TimeValue))
        bn_prop = next(g.subjects(KNORA_API.timeValueAsTimeStamp))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testTimeValue, bn_type))
        assert res_id == in_id


class TestUriValue:
    def test_make_graph_corr(self, rdf_uri_value_corr: UriValueRDF) -> None:
        in_id = URIRef("id")
        g = rdf_uri_value_corr.make_graph()
        assert len(g) == 3
        bn_type = next(g.subjects(RDF.type, KNORA_API.UriValue))
        bn_prop = next(g.subjects(KNORA_API.uriValueAsUri))
        assert bn_type == bn_prop
        res_id = next(g.subjects(ONTO.testUriValue, bn_type))
        assert res_id == in_id



import warnings

import pandas as pd
import pytest
import regex

from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_duplicates_all_excels
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_duplicate_custom_id_all_excels
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_duplicate_nodes_one_df
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_erroneous_entries_all_excels
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_erroneous_entries_one_column_level
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_erroneous_entries_one_grouped_df
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_erroneous_entries_one_list
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_erroneous_node_info_one_df
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_missing_translations_all_excels
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_missing_translations_one_column_group
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_missing_translations_one_sheet
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_for_unique_list_names
from dsp_tools.commands.excel2json.new_lists.compliance_checks import (
    _check_if_all_translations_in_all_column_levels_present_one_sheet,
)
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_missing_translations_one_row
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _check_warn_unusual_columns_one_sheet
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _make_shape_compliance_all_excels
from dsp_tools.commands.excel2json.new_lists.compliance_checks import _make_shape_compliance_one_sheet
from dsp_tools.commands.excel2json.new_lists.compliance_checks import make_all_excel_compliance_checks
from dsp_tools.commands.excel2json.new_lists.models.deserialise import Columns
from dsp_tools.commands.excel2json.new_lists.models.deserialise import ExcelSheet
from dsp_tools.commands.excel2json.new_lists.models.input_error import DuplicatesCustomIDInProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import DuplicatesInSheetProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import ListSheetComplianceProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import ListSheetContentProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import MinimumRowsProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import MissingExpectedColumn
from dsp_tools.commands.excel2json.new_lists.models.input_error import MissingNodeColumn
from dsp_tools.commands.excel2json.new_lists.models.input_error import MissingNodeTranslationProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import MissingTranslationsSheetProblem
from dsp_tools.commands.excel2json.new_lists.models.input_error import NodesPerRowProblem
from dsp_tools.models.custom_warnings import DspToolsUserWarning
from dsp_tools.models.exceptions import InputError


class TestMakeAllExcelComplianceChecks:
    def test_good(self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet) -> None:
        make_all_excel_compliance_checks([f1_s1_good_en, f2_s2_good_en_de])

    def test_duplicates(self, cols_en_1: Columns) -> None:
        df_1 = pd.DataFrame({"id (optional)": [1, 2], "en_list": ["list2", "list2"], "en_1": [pd.NA, 1]})
        df_2 = pd.DataFrame({"id (optional)": [1, 4], "en_list": ["list2", "list2"], "en_1": [pd.NA, 1]})
        all_sheets = [
            ExcelSheet(excel_name="file1", sheet_name="sheet2", df=df_1, col_info=cols_en_1),
            ExcelSheet(excel_name="file2", sheet_name="sheet2", df=df_2, col_info=cols_en_1),
        ]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "No duplicates are allowed in the 'ID (optional)' column. At the following locations, IDs are duplicated:"
            "\n----------------------------\n"
            "ID: '1'\n"
            "    - Excel 'file1' | Sheet 'sheet2' | Row 2\n"
            "    - Excel 'file2' | Sheet 'sheet2' | Row 2"
        )
        with pytest.raises(InputError, match=expected):
            make_all_excel_compliance_checks(all_sheets)

    def test_duplicate_list_names(self, cols_en_1_no_comments: Columns) -> None:
        df_1 = pd.DataFrame({"id (optional)": [1, 2], "en_list": ["list2", "list2"], "en_1": [pd.NA, 1]})
        df_2 = pd.DataFrame({"id (optional)": [3, 4], "en_list": ["list2", "list2"], "en_1": [pd.NA, 1]})
        all_sheets = [
            ExcelSheet(excel_name="file1", sheet_name="sheet2", df=df_1, col_info=cols_en_1_no_comments),
            ExcelSheet(excel_name="file2", sheet_name="sheet2", df=df_2, col_info=cols_en_1_no_comments),
        ]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The name of the list must be unique across all the excel sheets.\n"
            "The following sheets have lists with the same name:\n"
            "    - Excel file: 'file1', Sheet: 'sheet2', List: 'list2'\n"
            "    - Excel file: 'file2', Sheet: 'sheet2', List: 'list2'"
        )
        with pytest.raises(InputError, match=expected):
            make_all_excel_compliance_checks(all_sheets)

    def test_content_compliance(self, f1_s1_good_en: ExcelSheet, f2_s2_missing_translations: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_missing_translations]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file2' contains the following problems:\n\n"
            "The excel sheet 'sheet2' has the following problem(s):\n"
            "In one list, all the nodes must be translated into all the languages used. "
            "For the following nodes, the translations are missing:\n"
            "    - Row Number: 2 | Column(s): de_list\n"
            "    - Row Number: 3 | Column(s): en_1\n"
            "    - Row Number: 5 | Column(s): de_comments\n"
            "    - Row Number: 6 | Column(s): en_1, en_2, en_list"
        )
        with pytest.raises(InputError, match=expected):
            make_all_excel_compliance_checks(all_sheets)


class TestFormalExcelCompliance:
    def test_good(self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_good_en_de]
        _make_shape_compliance_all_excels(all_sheets)

    def test_problem(
        self, f1_s1_no_list_columns: ExcelSheet, f2_s2_missing_lang_column: ExcelSheet, f2_s3_one_row: ExcelSheet
    ) -> None:
        all_sheets = [f1_s1_no_list_columns, f2_s2_missing_lang_column, f2_s3_one_row]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file1' contains the following problems:\n\n"
            "The excel sheet 'sheet1' has the following problem(s):\n"
            "    - At least one column for the node names in the format '[lang]_[column_number]' is required. "
            "The allowed language tags are: en, de, fr, it, rm."
            "\n\n---------------------------------------\n\n"
            "The Excel file 'file2' contains the following problems:\n\n"
            "The excel sheet 'sheet2' has the following problem(s):\n"
            "    - All nodes and lists must be translated into the same languages. "
            "Based on the languages used, the following column(s) are missing: de_3"
            "\n----------------------------\n"
            "The excel sheet 'sheet3' has the following problem(s):\n"
            "    - The Excel sheet must contain at least two rows, "
            "one for the list name and one row for a minimum of one node."
        )

        with pytest.raises(InputError, match=expected):
            _make_shape_compliance_all_excels(all_sheets)


class TestCheckExcelsForDuplicates:
    def test_good(self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_good_en_de]
        _check_duplicates_all_excels(all_sheets)

    def test_problem(self, f1_s1_identical_row: ExcelSheet) -> None:
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file1' contains the following problems:\n\n"
            "The excel sheet 'sheet1' contains rows that are completely identical "
            "(excluding the column 'ID (optional)'). The following rows are duplicates:\n"
            "    - 3\n"
            "    - 4"
        )
        with pytest.raises(InputError, match=expected):
            _check_duplicates_all_excels([f1_s1_identical_row])

    def test_problem_duplicate_id(self, sheets_duplicate_id: list[ExcelSheet]) -> None:
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "No duplicates are allowed in the 'ID (optional)' column. At the following locations, IDs are duplicated:\n"
            "----------------------------\n"
            "ID: '1'\n"
            "    - Excel 'file1' | Sheet 'sheet1' | Row 3\n"
            "    - Excel 'file2' | Sheet 'sheet2' | Row 2\n"
            "----------------------------\n"
            "ID: '4'\n"
            "    - Excel 'file1' | Sheet 'sheet1' | Row 5\n"
            "    - Excel 'file2' | Sheet 'sheet2' | Row 4"
        )
        with pytest.raises(InputError, match=expected):
            _check_duplicates_all_excels(sheets_duplicate_id)


class TestCheckForDuplicateListNames:
    def test_good(self, cols_en_list_only: Columns) -> None:
        df_1 = pd.DataFrame({"en_list": ["list1", "list1"]})
        df_2 = pd.DataFrame({"en_list": ["list2", "list2"]})
        all_sheets = [
            ExcelSheet(excel_name="file1", sheet_name="sheet1", df=df_1, col_info=cols_en_list_only),
            ExcelSheet(excel_name="file2", sheet_name="sheet2", df=df_2, col_info=cols_en_list_only),
        ]
        _check_for_unique_list_names(all_sheets)

    def test_problem(self, cols_en_list_only: Columns) -> None:
        df_1 = pd.DataFrame({"en_list": ["list1", "list2"]})
        df_2 = pd.DataFrame({"en_list": ["list2", "list2"]})
        df_3 = pd.DataFrame({"en_list": ["list2", "list2"]})
        all_sheets = [
            ExcelSheet(excel_name="file1", sheet_name="sheet1", df=df_1, col_info=cols_en_list_only),
            ExcelSheet(excel_name="file1", sheet_name="sheet2", df=df_2, col_info=cols_en_list_only),
            ExcelSheet(excel_name="file2", sheet_name="sheet2", df=df_3, col_info=cols_en_list_only),
        ]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file1' contains the following problems:\n\n"
            "Per Excel sheet only one list is allowed.\n"
            "The sheet 'sheet1' has more than one list, namely the following: list1, list2"
            "\n\n---------------------------------------\n\n"
            "The name of the list must be unique across all the excel sheets.\n"
            "The following sheets have lists with the same name:\n"
            "    - Excel file: 'file1', Sheet: 'sheet1', List: 'list2'\n"
            "    - Excel file: 'file1', Sheet: 'sheet2', List: 'list2'\n"
            "    - Excel file: 'file2', Sheet: 'sheet2', List: 'list2'"
        )
        with pytest.raises(InputError, match=expected):
            _check_for_unique_list_names(all_sheets)


class TestCheckForDuplicates:
    def test_good(self, cols_en_1: Columns) -> None:
        test_df = pd.DataFrame({"en_list": ["a", "b", "c"], "en_1": ["d", "e", "f"]})
        test_sheet = ExcelSheet(excel_name="", sheet_name="sheet", df=test_df, col_info=cols_en_1)
        assert not _check_for_duplicate_nodes_one_df(test_sheet)

    def test_problem(self, cols_en_1_2: Columns) -> None:
        test_df = pd.DataFrame({"en_list": ["a", "a", "a"], "en_1": ["b", "b", "b"], "en_2": ["d", "c", "d"]})
        test_sheet = ExcelSheet(excel_name="excel", sheet_name="sheet", df=test_df, col_info=cols_en_1_2)
        res = _check_for_duplicate_nodes_one_df(test_sheet)
        assert isinstance(res, DuplicatesInSheetProblem)
        assert res.excel_name == "excel"
        assert res.sheet_name == "sheet"
        assert res.rows == [0, 2]

    def test_check_for_duplicate_custom_id_all_excels_good(
        self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet
    ) -> None:
        sheets = [f1_s1_good_en, f2_s2_good_en_de]
        assert not _check_for_duplicate_custom_id_all_excels(sheets)

    def test_check_for_duplicate_custom_id_all_excels_problem(self, sheets_duplicate_id: list[ExcelSheet]) -> None:
        result = _check_for_duplicate_custom_id_all_excels(sheets_duplicate_id)
        assert isinstance(result, DuplicatesCustomIDInProblem)
        assert len(result.duplicate_ids) == 2
        one = result.duplicate_ids[0]
        assert one.custom_id == 1
        assert len(one.excel_locations) == 2
        locations = sorted(one.excel_locations, key=lambda x: x.excel_filename)  # type: ignore[return-value,arg-type]
        assert locations[0].excel_filename == "file1"
        assert locations[0].sheet == "sheet1"
        assert locations[0].row == 3
        assert locations[1].excel_filename == "file2"
        assert locations[1].sheet == "sheet2"
        assert locations[1].row == 2
        two = result.duplicate_ids[1]
        assert two.custom_id == 4
        assert len(two.excel_locations) == 2
        locations = sorted(two.excel_locations, key=lambda x: x.excel_filename)  # type: ignore[return-value,arg-type]
        assert locations[0].excel_filename == "file1"
        assert locations[0].sheet == "sheet1"
        assert locations[0].row == 5
        assert locations[1].excel_filename == "file2"
        assert locations[1].sheet == "sheet2"
        assert locations[1].row == 4


class TestShapeCompliance:
    def test_good(self, cols_en_1: Columns) -> None:
        test_df = pd.DataFrame({"id (optional)": [1, 2, 3], "en_list": ["a", "b", "c"], "en_1": ["d", "e", "f"]})
        test_sheet = ExcelSheet(excel_name="", sheet_name="sheet", df=test_df, col_info=cols_en_1)
        assert not _make_shape_compliance_one_sheet(test_sheet)

    def test_good_no_id(self, cols_en_1: Columns) -> None:
        test_df = pd.DataFrame({"en_list": ["a", "b", "c"], "en_1": ["d", "e", "f"]})
        test_sheet = ExcelSheet(excel_name="", sheet_name="sheet", df=test_df, col_info=cols_en_1)
        assert not _make_shape_compliance_one_sheet(test_sheet)

    def test_missing_node_column(self, cols_en_list_only: Columns) -> None:
        test_df = pd.DataFrame({"id (optional)": [1, 1], "en_list": ["a", "b"], "additional_1": ["b", "c"]})
        test_sheet = ExcelSheet(excel_name="excel", sheet_name="sheet", df=test_df, col_info=cols_en_list_only)
        warning_msg = regex.escape(
            "The following columns do not conform to the expected format "
            "and will not be included in the output: additional_1"
        )
        with pytest.warns(DspToolsUserWarning, match=warning_msg):
            res = _make_shape_compliance_one_sheet(test_sheet)
            assert isinstance(res, ListSheetComplianceProblem)
            assert res.excel_name == "excel"
            assert res.sheet_name == "sheet"
            assert len(res.problems) == 1
            assert isinstance(res.problems[0], MissingNodeColumn)

    def test_missing_list_column(self, cols_en_1: Columns) -> None:
        test_df = pd.DataFrame({"id (optional)": [1, 2], "en_1": ["b", "c"]})
        test_sheet = ExcelSheet(excel_name="excel", sheet_name="sheet", df=test_df, col_info=cols_en_1)
        res = _make_shape_compliance_one_sheet(test_sheet)
        assert isinstance(res, ListSheetComplianceProblem)
        assert res.excel_name == "excel"
        assert res.sheet_name == "sheet"
        assert len(res.problems) == 1
        missing = res.problems[0]
        assert isinstance(missing, MissingExpectedColumn)
        assert missing.missing_cols == {"en_list"}

    def test_missing_translation_column(self, cols_en_de_1: Columns) -> None:
        test_df = pd.DataFrame(
            {"id (optional)": [1, 2], "en_list": ["a", "b"], "de_list": ["b", "c"], "de_1": ["b", "c"]}
        )
        test_sheet = ExcelSheet(excel_name="excel", sheet_name="sheet", df=test_df, col_info=cols_en_de_1)
        res = _make_shape_compliance_one_sheet(test_sheet)
        assert isinstance(res, ListSheetComplianceProblem)
        assert res.excel_name == "excel"
        assert res.sheet_name == "sheet"
        assert len(res.problems) == 1
        missing = res.problems[0]
        assert isinstance(missing, MissingExpectedColumn)
        assert missing.missing_cols == {"en_1"}

    def test_missing_rows(self, cols_en_1: Columns) -> None:
        test_df = pd.DataFrame({"id (optional)": [1], "en_list": ["a"], "en_1": ["d"]})
        test_sheet = ExcelSheet(excel_name="", sheet_name="sheet", df=test_df, col_info=cols_en_1)
        res = _make_shape_compliance_one_sheet(test_sheet)
        assert isinstance(res, ListSheetComplianceProblem)
        assert len(res.problems) == 1
        assert isinstance(res.problems[0], MinimumRowsProblem)


class TestCheckWarnUnusualColumns:
    def test_good(self) -> None:
        test_cols = pd.Index(["id (optional)", "en_list", "en_2", "de_2"])
        with warnings.catch_warnings(record=True) as caught_warnings:
            _check_warn_unusual_columns_one_sheet(test_cols)
        assert len(caught_warnings) == 0

    def test_additional_columns(self) -> None:
        test_cols = pd.Index(["id (optional)", "en_list", "en_2", "de_2", "additional_1", "additional_2"])
        expected = regex.escape(
            "The following columns do not conform to the expected format "
            "and will not be included in the output: additional_1, additional_2"
        )
        with pytest.warns(DspToolsUserWarning, match=expected):
            _check_warn_unusual_columns_one_sheet(test_cols)


class TestCheckAllTranslationsPresent:
    def test_good(self) -> None:
        test_cols = pd.Index(["id (optional)", "en_list", "de_list", "de_1", "en_1", "de_2", "en_2"])
        assert not _check_if_all_translations_in_all_column_levels_present_one_sheet(test_cols)

    def test_missing_translations_node_columns(self) -> None:
        test_cols = pd.Index(["id (optional)", "en_list", "de_list", "de_1", "en_1", "de_2"])
        res = _check_if_all_translations_in_all_column_levels_present_one_sheet(test_cols)
        assert isinstance(res, MissingExpectedColumn)
        assert res.missing_cols == {"en_2"}

    def test_missing_translations_list_columns(self) -> None:
        test_cols = pd.Index(["id (optional)", "en_list", "de_1", "en_1", "de_2", "en_2"])
        res = _check_if_all_translations_in_all_column_levels_present_one_sheet(test_cols)
        assert isinstance(res, MissingExpectedColumn)
        assert res.missing_cols == {"de_list"}


class TestCheckAllExcelsMissingTranslations:
    def test_good(self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_good_en_de]
        _check_for_missing_translations_all_excels(all_sheets)

    def test_problem(self, f1_s1_good_en: ExcelSheet, f2_s2_missing_translations: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_missing_translations]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file2' contains the following problems:\n\n"
            "The excel sheet 'sheet2' has the following problem(s):\n"
            "In one list, all the nodes must be translated into all the languages used. "
            "For the following nodes, the translations are missing:\n"
            "    - Row Number: 2 | Column(s): de_list\n"
            "    - Row Number: 3 | Column(s): en_1\n"
            "    - Row Number: 5 | Column(s): de_comments\n"
            "    - Row Number: 6 | Column(s): en_1, en_2, en_list"
        )
        with pytest.raises(InputError, match=expected):
            _check_for_missing_translations_all_excels(all_sheets)


class TestAllNodesTranslatedIntoAllLanguages:
    def test_good(self, f1_s1_good_id_filled_empty_comments: ExcelSheet) -> None:
        assert not _check_for_missing_translations_one_sheet(f1_s1_good_id_filled_empty_comments)

    def test_missing_translation(self, f1_s1_missing_translation_id_filled: ExcelSheet) -> None:
        expected = [
            MissingNodeTranslationProblem(["de_2"], 2),
            MissingNodeTranslationProblem(["en_list"], 5),
            MissingNodeTranslationProblem(["en_1"], 8),
        ]
        result = _check_for_missing_translations_one_sheet(f1_s1_missing_translation_id_filled)
        assert isinstance(result, MissingTranslationsSheetProblem)
        assert result.excel_name == "excel1"
        assert result.sheet == "sheet1"
        res_node_problems = sorted(result.node_problems, key=lambda x: x.index_num)
        for res, expct in zip(res_node_problems, expected):
            assert res.empty_columns == expct.empty_columns
            assert res.index_num == expct.index_num


class TestCheckMissingTranslationsOneRow:
    def test_good(self, f2_s2_good_en_de: ExcelSheet, cols_en_de_1_3: Columns) -> None:
        for i, row in f2_s2_good_en_de.df.iterrows():
            assert not _check_missing_translations_one_row(int(str(i)), row, cols_en_de_1_3)

    def test_good_empty_comments(
        self, f1_s1_good_id_filled_empty_comments: ExcelSheet, cols_en_de_1_3: Columns
    ) -> None:
        for i, row in f1_s1_good_id_filled_empty_comments.df.iterrows():
            assert not _check_missing_translations_one_row(int(str(i)), row, cols_en_de_1_3)

    def test_one_missing(self, f2_s2_missing_translations: ExcelSheet, cols_en_de_1_3: Columns) -> None:
        result = _check_missing_translations_one_row(1, f2_s2_missing_translations.df.loc[1], cols_en_de_1_3)  # type: ignore[arg-type]
        assert isinstance(result, MissingNodeTranslationProblem)
        assert set(result.empty_columns) == {"en_1"}
        assert result.index_num == 1

    def test_three_missing(self, f2_s2_missing_translations: ExcelSheet, cols_en_de_1_3: Columns) -> None:
        result = _check_missing_translations_one_row(4, f2_s2_missing_translations.df.loc[4], cols_en_de_1_3)  # type: ignore[arg-type]
        assert isinstance(result, MissingNodeTranslationProblem)
        assert set(result.empty_columns) == {"en_list", "en_1", "en_2"}
        assert result.index_num == 4

    def test_one_group_good(self, f2_s2_good_en_de: ExcelSheet) -> None:
        for i, row in f2_s2_good_en_de.df.iterrows():
            assert not _check_for_missing_translations_one_column_group(row, ["en_list", "de_list"])

    def test_one_group_missing(self) -> None:
        series = pd.Series(data=[pd.NA, "content"], index=["en_1", "de_1"])
        result = _check_for_missing_translations_one_column_group(series, ["en_1", "de_1"])
        assert result == ["en_1"]


class TestCheckAllExcelForRowProblems:
    def test_all_good(self, f1_s1_good_en: ExcelSheet, f2_s2_good_en_de: ExcelSheet) -> None:
        all_sheets = [f1_s1_good_en, f2_s2_good_en_de]
        _check_for_erroneous_entries_all_excels(all_sheets)

    def test_all_problem(self, cols_en_1: Columns, cols_en_1_2: Columns) -> None:
        df_1 = pd.DataFrame({"en_list": ["list1", "list1", "list1", "list1"], "en_1": [pd.NA, "node1", pd.NA, "node3"]})
        df_2 = pd.DataFrame(
            {
                "en_list": ["list1", "list1", "list1", "list1", "list1", "list1"],
                "en_1": [pd.NA, "node1", "node1", "node1", "node2", "node3"],
                "en_2": [pd.NA, pd.NA, "node1.1", "node1.2", "node2.1", pd.NA],
            }
        )
        all_sheets = [
            ExcelSheet(excel_name="file1", sheet_name="sheet1", df=df_1, col_info=cols_en_1),
            ExcelSheet(excel_name="file2", sheet_name="sheet2", df=df_2, col_info=cols_en_1_2),
        ]
        expected = regex.escape(
            "\nThe excel file(s) used to create the list section have the following problem(s):\n\n"
            "The Excel file 'file1' contains the following problems:\n\n"
            "The Excel sheet 'sheet1' has the following problem(s):\n"
            "    - Row Number: 4, Column(s) that must be filled: en_1\n\n"
            "---------------------------------------\n\n"
            "The Excel file 'file2' contains the following problems:\n\n"
            "The Excel sheet 'sheet2' has the following problem(s):\n"
            "    - Row Number: 6, Column(s) that must be empty: en_2"
        )
        with pytest.raises(InputError, match=expected):
            _check_for_erroneous_entries_all_excels(all_sheets)


class TestOneSheetErrors:
    def test_all_good_flat(self, cols_en_1: Columns) -> None:
        df = pd.DataFrame({"en_list": ["list1", "list1", "list1", "list1"], "en_1": [pd.NA, "node1", "node2", "node3"]})
        test_sheet = ExcelSheet(excel_name="", sheet_name="sheet", df=df, col_info=cols_en_1)
        assert not _check_for_erroneous_entries_one_list(test_sheet)

    def test_problem(self, cols_en_1_2: Columns) -> None:
        df = pd.DataFrame(
            {
                "en_list": ["list1", "list1", "list1", "list1", "list1", "list1"],
                "en_1": [pd.NA, "node1", "node1", "node1", "node2", "node3"],
                "en_2": [pd.NA, pd.NA, "node1.1", "node1.2", "node2.1", pd.NA],
            }
        )
        test_sheet = ExcelSheet(excel_name="excel_name", sheet_name="sheet", df=df, col_info=cols_en_1_2)
        res = _check_for_erroneous_entries_one_list(test_sheet)
        assert isinstance(res, ListSheetContentProblem)
        assert res.excel_name == "excel_name"
        assert res.sheet_name == "sheet"
        assert len(res.problems) == 1


class TestCheckForErroneousEntries:
    def test_all_good_subnodes(self, f2_s2_good_en_de: ExcelSheet) -> None:
        assert not _check_for_erroneous_node_info_one_df(f2_s2_good_en_de.df, ["en_list", "en_1", "en_2", "en_3"])

    def test_all_good_flat(self, f1_s1_good_en: ExcelSheet) -> None:
        assert not _check_for_erroneous_node_info_one_df(f1_s1_good_en.df, ["en_list", "en_1"])

    def test_all_good_duplicate_names(self) -> None:
        df = pd.DataFrame(
            {
                "en_list": ["list1", "list1", "list1", "list1", "list1", "list1", "list1"],
                "en_1": [pd.NA, "node1", "node1", "node2", "node2", "node2", "node2"],
                "en_2": [pd.NA, pd.NA, "same", pd.NA, "same", "2.1", "2.1"],
                "en_3": [pd.NA, pd.NA, pd.NA, pd.NA, pd.NA, pd.NA, "2.1.1"],
            }
        )
        assert not _check_for_erroneous_node_info_one_df(df, ["en_list", "en_1", "en_2", "en_3"])

    def test_missing_row(self) -> None:
        df = pd.DataFrame(
            {
                "en_list": ["list1", "list1", "list1", "list1", "list1", "list1"],
                "en_1": [pd.NA, "node1", "node1", "node1", "node2", "node3"],
                "en_2": [pd.NA, pd.NA, "node1.1", "node1.2", "node2.1", pd.NA],
                "en_3": [pd.NA, pd.NA, "node1.1.1", pd.NA, pd.NA, pd.NA],
            }
        )
        # "node1.1" & "node2" is missing
        res = _check_for_erroneous_node_info_one_df(df, ["en_list", "en_1", "en_2", "en_3"])
        res = sorted(res, key=lambda x: x.index_num)
        assert len(res) == 2
        assert isinstance(res[0], NodesPerRowProblem)
        assert res[0].column_names == ["en_3"]
        assert res[0].index_num == 2
        assert res[0].should_be_empty
        assert isinstance(res[1], NodesPerRowProblem)
        assert res[1].column_names == ["en_2", "en_3"]
        assert res[1].index_num == 4
        assert res[1].should_be_empty


class TestCheckOneColumnGroupsForErroneousEntries:
    def test_good(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "b", "c"], "two": [pd.NA, "bb", "bb", "cc"], "other": ["a", "b", pd.NA, pd.NA]},
            index=[2, 3, 4, 5],
        )
        assert not _check_for_erroneous_entries_one_column_level(df, ["two"], 0)

    def test_missing(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "b", "c"], "two": [pd.NA, "bb", pd.NA, pd.NA], "other": ["a", "b", pd.NA, pd.NA]},
            index=[2, 3, 4, 5],
        )
        res = _check_for_erroneous_entries_one_column_level(df, ["one", "two"], 0)
        assert len(res) == 2
        assert isinstance(res[0], NodesPerRowProblem)
        assert res[0].column_names == ["two"]
        assert res[0].index_num == 3
        assert res[0].should_be_empty
        assert isinstance(res[1], NodesPerRowProblem)
        assert res[1].column_names == ["two"]
        assert res[1].index_num == 4
        assert not res[1].should_be_empty


class TestCheckOneGroupForErroneousEntries:
    def test_good(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "c"], "two": [pd.NA, "bb", "cc"], "other": ["a", "b", pd.NA]}, index=[2, 3, 4]
        )
        assert not _check_for_erroneous_entries_one_grouped_df(df, ["one", "two"])

    def test_good_one_col(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "c"], "two": [pd.NA, "bb", "cc"], "other": ["a", "b", pd.NA]}, index=[2, 3, 4]
        )
        assert not _check_for_erroneous_entries_one_grouped_df(df, ["two"])

    def test_filled_first_row(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "c"], "two": ["filled", "bb", "cc"], "other": ["a", "b", pd.NA]}, index=[2, 3, 4]
        )
        res = _check_for_erroneous_entries_one_grouped_df(df, ["one", "two"])
        assert len(res) == 1
        assert isinstance(res[0], NodesPerRowProblem)
        assert res[0].column_names == ["two"]
        assert res[0].index_num == 2
        assert res[0].should_be_empty

    def test_missing_next_rows(self) -> None:
        df = pd.DataFrame(
            {"one": ["a", "b", "c", "d"], "two": [pd.NA, pd.NA, "cc", pd.NA], "other": ["a", "b", pd.NA, "d"]},
            index=[2, 3, 4, 5],
        )
        res = _check_for_erroneous_entries_one_grouped_df(df, ["one", "two"])
        assert len(res) == 2
        assert isinstance(res[0], NodesPerRowProblem)
        assert res[0].column_names == ["two"]
        assert res[0].index_num == 3
        assert not res[0].should_be_empty
        assert isinstance(res[1], NodesPerRowProblem)
        assert res[1].column_names == ["two"]
        assert res[1].index_num == 5
        assert not res[1].should_be_empty


if __name__ == "__main__":
    pytest.main([__file__])



import pytest

from dsp_tools.commands.xmlupload.models.namespace_context import get_default_json_ld_context
from dsp_tools.commands.xmlupload.models.namespace_context import get_json_ld_context_for_project


def test_get_default_context() -> None:
    context = get_default_json_ld_context()
    expected = {
        "knora-api": "http://api.knora.org/ontology/knora-api/v2#",
        "salsah-gui": "http://api.knora.org/ontology/salsah-gui/v2#",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "owl": "http://www.w3.org/2002/07/owl#",
        "xsd": "http://www.w3.org/2001/XMLSchema#",
    }
    assert context == expected


def test_get_json_ld_context_for_project() -> None:
    context = get_json_ld_context_for_project({"testonto": "http://www.knora.org/ontology/testonto"})
    expected = {
        "knora-api": "http://api.knora.org/ontology/knora-api/v2#",
        "salsah-gui": "http://api.knora.org/ontology/salsah-gui/v2#",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "owl": "http://www.w3.org/2002/07/owl#",
        "xsd": "http://www.w3.org/2001/XMLSchema#",
        "testonto": "http://www.knora.org/ontology/testonto#",
    }
    assert context == expected


def test_get_json_ld_context_for_project_with_multiple_ontologies() -> None:
    context = get_json_ld_context_for_project(
        {
            "testonto": "http://www.knora.org/ontology/testonto",
            "testonto2": "http://www.knora.org/ontology/testonto2",
        }
    )
    expected = {
        "knora-api": "http://api.knora.org/ontology/knora-api/v2#",
        "salsah-gui": "http://api.knora.org/ontology/salsah-gui/v2#",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "owl": "http://www.w3.org/2002/07/owl#",
        "xsd": "http://www.w3.org/2001/XMLSchema#",
        "testonto": "http://www.knora.org/ontology/testonto#",
        "testonto2": "http://www.knora.org/ontology/testonto2#",
    }
    assert context == expected


if __name__ == "__main__":
    pytest.main([__file__])
